"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.login = void 0;
const http_1 = __importDefault(require("http"));
const open_1 = __importDefault(require("open"));
const utils_1 = require("../utils");
const configuration_1 = __importDefault(require("../configuration"));
const spinner_1 = __importDefault(require("../outputs/spinner"));
const spheron_api_1 = __importDefault(require("../services/spheron-api"));
let server;
async function login(provider) {
    const spinner = new spinner_1.default();
    spinner.spin(`Waiting for ${provider} authentication to be completed `);
    server = http_1.default.createServer();
    server.listen(0, "127.0.0.1", async () => {
        const { port } = server.address();
        const baseURL = `${configuration_1.default.spheronServerAddress}/auth/${provider}/cli/login`;
        const fullURL = baseURL + `?port=${port}`;
        let loginError = false;
        try {
            await Promise.all([
                new Promise((resolve, reject) => {
                    server.once("request", async (req, res) => {
                        var _a, _b, _c, _d, _e;
                        try {
                            const code = (_a = req.url) === null || _a === void 0 ? void 0 : _a.split("&")[0].split("=")[1];
                            const verify = await spheron_api_1.default.verfiyGitToken(provider, String(code), port);
                            res.setHeader("connection", "close");
                            res.statusCode = 302;
                            const successLocationRedirect = new URL(`${configuration_1.default.spheronFrontendAddress}/#/notifications/cli-login?email=${encodeURIComponent(verify.email)}`);
                            res.setHeader("location", successLocationRedirect.href);
                            res.end();
                            await (0, utils_1.writeToJsonFile)("jwtToken", verify.jwtToken, configuration_1.default.configFilePath);
                            await (0, utils_1.writeToJsonFile)("organization", verify.organizationId, configuration_1.default.configFilePath);
                            resolve();
                        }
                        catch (e) {
                            if ((_c = (_b = e.response) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.message) {
                                const error_message = (_e = (_d = e.response) === null || _d === void 0 ? void 0 : _d.data) === null || _e === void 0 ? void 0 : _e.message;
                                if (error_message === "security_check") {
                                    console.log(`Your account is created! Before using our platform go ahead and fill up the form for security reasons. Visit ${configuration_1.default.spheronFrontendAddress} for more information.`);
                                }
                                else if (error_message === "banned") {
                                    console.log(`Your account has been banned. Before using our platform again please fill up security form. Visit ${configuration_1.default.spheronFrontendAddress} for more information.`);
                                }
                                else if (error_message === "user_not_found") {
                                    console.log("User not found.");
                                }
                                else if (error_message === "ask_admin_to_provide_access_to_dev_env") {
                                    console.log("You signup request is in review process. Please contact Spheron team for more information.");
                                }
                                else if (error_message === "waiting_for_access_to_dev_env") {
                                    console.log("You signup request is in review process. Please contact Spheron team for more information.");
                                }
                            }
                            console.log(`✖️  Error occured while logging in, please try again.`);
                            spinner.stop();
                            process.exit(0);
                        }
                    });
                    server.once("error", reject);
                }),
                (0, open_1.default)(fullURL),
            ]);
        }
        catch (error) {
            loginError = true;
        }
        finally {
            server.close();
            if (loginError) {
                console.log(`✖️  Error occured while logging in`);
            }
            else {
                spinner.success(`${provider} authentication completed`);
            }
            spinner.stop();
            process.exit(0);
        }
    });
}
exports.login = login;
