"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.publish = void 0;
const path_1 = __importDefault(require("path"));
const configuration_1 = __importDefault(require("../configuration"));
const utils_1 = require("../utils");
const upload_1 = require("./upload");
async function publish(organization) {
    var _a, _b;
    try {
        const localJsonPath = "./spheron.json";
        const projectConfig = await (0, utils_1.fileExists)(localJsonPath);
        if (!projectConfig) {
            throw new Error("spheron.json file does not exist. Please create it with spheron init command");
        }
        const spheronConfig = await (0, utils_1.fileExists)(configuration_1.default.configFilePath);
        if (!spheronConfig) {
            throw new Error(`Global configuration does not exist. Please execute spheron init command.`);
        }
        const localConfiguration = await (0, utils_1.readFromJsonFile)("configuration", localJsonPath);
        const organizationId = organization
            ? organization
            : await (0, utils_1.readFromJsonFile)("organization", configuration_1.default.configFilePath);
        if (!organizationId) {
            throw new Error("Please specify organization that you would wish to use while uploading");
        }
        const publishDirectory = (_b = (_a = localConfiguration === null || localConfiguration === void 0 ? void 0 : localConfiguration.framework) === null || _a === void 0 ? void 0 : _a.configuration) === null || _b === void 0 ? void 0 : _b.publishDirectory;
        if (publishDirectory == undefined || publishDirectory == null) {
            throw new Error("Publish directory not provided, please execute spheron init command");
        }
        const pathToDir = path_1.default.join(localConfiguration.rootPath, publishDirectory);
        if (!(await (0, utils_1.fileExists)(pathToDir))) {
            throw new Error(`Publish directory ${pathToDir} does not exist`);
        }
        console.log(`Publishing your dapp to ${(0, utils_1.mapProtocolToUserReadable)(localConfiguration.protocol)} 🚀`);
        await (0, upload_1.upload)(pathToDir, localConfiguration.protocol, organizationId, localConfiguration.name);
    }
    catch (error) {
        console.log(`✖️  Error: ${error.message}`);
        throw error;
    }
}
exports.publish = publish;
