"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.upload = void 0;
const configuration_1 = __importDefault(require("../configuration"));
const storage_1 = __importStar(require("@spheron/storage"));
const chalk = require("chalk");
const utils_1 = require("../utils");
const spinner_1 = __importDefault(require("../outputs/spinner"));
const progress_bar_1 = require("../outputs/progress-bar");
async function upload(rootPath, protocol, organizationId, projectName) {
    const spinner = new spinner_1.default();
    try {
        spinner.spin(`Uploading to ${(0, utils_1.mapProtocolToUserReadable)(protocol)} `, 4000);
        const jwtToken = await (0, utils_1.readFromJsonFile)("jwtToken", configuration_1.default.configFilePath);
        if (!jwtToken) {
            throw new Error("Authorization failed. Please execute login command first");
        }
        const client = new storage_1.default({ token: jwtToken });
        let uploadedBytes = 0;
        let uploadStarted = false;
        if (!(await (0, utils_1.fileExists)(rootPath))) {
            throw new Error(`File ${rootPath} does not exist`);
        }
        const fileType = await (0, utils_1.getFileType)(rootPath);
        console.log(`Uploading ${fileType} ${rootPath}`);
        const { uploadId, bucketId, protocolLink, dynamicLinks } = await client.upload(rootPath, {
            protocol: mapProtocolToStorageSDK(protocol),
            name: projectName,
            organizationId,
            onUploadInitiated: (deploymentId) => {
                console.log("Upload started, ID of deployment: " + deploymentId);
            },
            onChunkUploaded(uploadedSize, totalSize) {
                if (!uploadStarted) {
                    uploadStarted = true;
                    console.log((0, progress_bar_1.progressBar)(0, totalSize));
                    process.stdout.moveCursor(0, -1);
                }
                else {
                    uploadedBytes += uploadedSize;
                    process.stdout.moveCursor(0, 1);
                    process.stdout.clearLine(-1);
                    process.stdout.cursorTo(0);
                    console.log(`Uploaded ${formatBytes(uploadedBytes)}/${formatBytes(totalSize)} (${formatPercentage(uploadedBytes, totalSize)}), ${(0, progress_bar_1.progressBar)(uploadedBytes, totalSize)}`);
                    process.stdout.moveCursor(0, -2);
                }
            },
        });
        spinner.success("Upload finished !");
        spinner.stop();
        console.log("Here are upload details:");
        console.log(`Upload ID: ${uploadId}`);
        console.log(`Bucket ID: ${bucketId}`);
        console.log(chalk.green(`Protocol Link:`), protocolLink);
        console.log(chalk.green(`Dynamic Links:`), dynamicLinks
            .map((link) => {
            return `https://${link}`;
        })
            .join(", "));
    }
    catch (error) {
        console.log(`✖️  Error: ${error.message}`);
        if (protocol == "arweave") {
            console.log(`Arweave is not enabled for starter plan, please upgrade your plan to pro to start uploading your files to Arweave.\nPlease feel free to contact our team on Discord if you have any questions.`);
        }
    }
    finally {
        spinner.stop();
    }
}
exports.upload = upload;
function mapProtocolToStorageSDK(protocol) {
    if (protocol === "ipfs") {
        return storage_1.ProtocolEnum.IPFS;
    }
    else if (protocol === "filecoin") {
        return storage_1.ProtocolEnum.FILECOIN;
    }
    else if (protocol === "arweave") {
        return storage_1.ProtocolEnum.ARWEAVE;
    }
    return storage_1.ProtocolEnum.IPFS;
}
function formatBytes(bytes) {
    const units = ["B", "KB", "MB", "GB", "TB"];
    let index = 0;
    while (bytes >= 1024 && index < units.length - 1) {
        bytes /= 1024;
        index++;
    }
    return `${bytes.toFixed(2)} ${units[index]}`;
}
function formatPercentage(value, total) {
    const percentage = (value / total) * 100;
    return `${percentage.toFixed(2)}%`;
}
